@@ -1,6 +1,6 @@
-/* Copyright (c) 2015-2017, 2019 The Khronos Group Inc.
- * Copyright (c) 2015-2017, 2019 Valve Corporation
- * Copyright (c) 2015-2017, 2019 LunarG, Inc.
+/* Copyright (c) 2015-2017, 2019-2020 The Khronos Group Inc.
+ * Copyright (c) 2015-2017, 2019-2020 Valve Corporation
+ * Copyright (c) 2015-2017, 2019-2020 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -136,6 +136,47 @@ VK_LAYER_EXPORT VkLayerDeviceCreateInfo *get_chain_info(const VkDeviceCreateInfo
 
 static inline bool IsPowerOfTwo(unsigned x) { return x && !(x & (x - 1)); }
 
+static inline uint32_t SampleCountSize(VkSampleCountFlagBits sample_count) {
+    uint32_t size = 0;
+    switch (sample_count) {
+        case VK_SAMPLE_COUNT_1_BIT:
+            size = 1;
+            break;
+        case VK_SAMPLE_COUNT_2_BIT:
+            size = 2;
+            break;
+        case VK_SAMPLE_COUNT_4_BIT:
+            size = 4;
+            break;
+        case VK_SAMPLE_COUNT_8_BIT:
+            size = 8;
+            break;
+        case VK_SAMPLE_COUNT_16_BIT:
+            size = 16;
+            break;
+        case VK_SAMPLE_COUNT_32_BIT:
+            size = 32;
+            break;
+        case VK_SAMPLE_COUNT_64_BIT:
+            size = 64;
+            break;
+        default:
+            size = 0;
+    }
+    return size;
+}
+
+static inline bool IsIdentitySwizzle(VkComponentMapping components) {
+    // clang-format off
+    return (
+        ((components.r == VK_COMPONENT_SWIZZLE_IDENTITY) || (components.r == VK_COMPONENT_SWIZZLE_R)) &&
+        ((components.g == VK_COMPONENT_SWIZZLE_IDENTITY) || (components.g == VK_COMPONENT_SWIZZLE_G)) &&
+        ((components.b == VK_COMPONENT_SWIZZLE_IDENTITY) || (components.b == VK_COMPONENT_SWIZZLE_B)) &&
+        ((components.a == VK_COMPONENT_SWIZZLE_IDENTITY) || (components.a == VK_COMPONENT_SWIZZLE_A))
+    );
+    // clang-format on
+}
+
 extern "C" {
 #endif
 
@@ -173,14 +214,16 @@ static inline int u_ffs(int val) {
 }
 #endif
 
-// shared_mutex support added in MSVC 2015 update 2
-#if defined(_MSC_FULL_VER) && _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2
+// Minimum Visual Studio 2015 Update 2, or libc++ with C++17
+#if defined(_MSC_FULL_VER) && _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2 && \
+    (!defined(_LIBCPP_VERSION) || __cplusplus >= 201703)
 #include <shared_mutex>
 #endif
 
 class ReadWriteLock {
   private:
-#if defined(_MSC_FULL_VER) && _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2
+#if defined(_MSC_FULL_VER) && _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2 && \
+    (!defined(_LIBCPP_VERSION) || __cplusplus >= 201703)
     typedef std::shared_mutex lock_t;
 #else
     typedef std::mutex lock_t;
@@ -190,7 +233,8 @@ class ReadWriteLock {
     void lock() { m_lock.lock(); }
     bool try_lock() { return m_lock.try_lock(); }
     void unlock() { m_lock.unlock(); }
-#if defined(_MSC_FULL_VER) && _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2
+#if defined(_MSC_FULL_VER) && _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2 && \
+    (!defined(_LIBCPP_VERSION) || __cplusplus >= 201703)
     void lock_shared() { m_lock.lock_shared(); }
     bool try_lock_shared() { return m_lock.try_lock_shared(); }
     void unlock_shared() { m_lock.unlock_shared(); }
@@ -203,7 +247,8 @@ class ReadWriteLock {
     lock_t m_lock;
 };
 
-#if defined(_MSC_FULL_VER) && _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2
+#if defined(_MSC_FULL_VER) && _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2 && \
+    (!defined(_LIBCPP_VERSION) || __cplusplus >= 201703)
 typedef std::shared_lock<ReadWriteLock> read_lock_guard_t;
 typedef std::unique_lock<ReadWriteLock> write_lock_guard_t;
 #else
